package storage

import (
	"context"
	"fmt"
	"sync"

	"git.espin.casa/albert/cml04-falcon-system/internal/types"
	"gorm.io/driver/postgres"
	"gorm.io/gorm"
)

type Storager interface {
	StoreBarcode(ctx context.Context, in *types.Barcode) error
	StoreProductionOrder(ctx context.Context, in *types.ProductionOrder) error
	StoreCustomerOrder(ctx context.Context, in *types.CustomerOrder) error
	StoreBeamCutPattern(ctx context.Context, in *types.BeamCutPattern) error
	StoreLayerCutPattern(ctx context.Context, in *types.LayerCutPattern) error
	StoreBeamGroup(ctx context.Context, in *types.BeamGroup) error
	StoreBundle(ctx context.Context, in *types.BundleData) error
	ConfirmBundle(ctx context.Context, barcode string) error
}

type storage struct {
	db  *gorm.DB
	mux sync.RWMutex
}

type DBConfig struct {
	Username string
	Password string
	Host     string
	Port     int
	Name     string
}

// StoreBundle implements Storager.
func (s *storage) StoreBundle(ctx context.Context, in *types.BundleData) error {
	return s.db.Save(in).Error
}

// StoreBeamGroup implements Storager.
func (s *storage) StoreBeamGroup(ctx context.Context, in *types.BeamGroup) error {
	return s.db.Save(in).Error
}

// StoreLayerCutPattern implements Storager.
func (s *storage) StoreLayerCutPattern(ctx context.Context, in *types.LayerCutPattern) error {
	return s.db.Save(in).Error
}

// StoreBeamCutPattern implements Storager.
func (s *storage) StoreBeamCutPattern(ctx context.Context, in *types.BeamCutPattern) error {
	return s.db.Save(in).Error
}

// StoreCustomerOrder implements Storager.
func (s *storage) StoreCustomerOrder(ctx context.Context, in *types.CustomerOrder) error {
	return s.db.Save(in).Error
}

// StoreProductionOrder implements Storager.
func (s *storage) StoreProductionOrder(ctx context.Context, in *types.ProductionOrder) error {
	return s.db.Save(in).Error
}

// ConfirmBundle implements Storager.
func (s *storage) ConfirmBundle(ctx context.Context, barcode string) error {
	bc := &types.Barcode{}
	if err := s.db.First(&bc, barcode).Error; err != nil {
		return err
	}
	return s.db.Model(&bc).Where("nromatricula = ?", barcode).Update("confirmed", true).Error
}

// StoreBarcode implements IStorage.
func (s *storage) StoreBarcode(ctx context.Context, in *types.Barcode) error {
	return s.db.Save(in).Error
}

func ProductionDataBase(conf *DBConfig) (*gorm.DB, error) {
	// create dsn string
	dsn := fmt.Sprintf("host=%s user=%s password=%s dbname=%s port=%d sslmode=disable TimeZone=Europe/Madrid",
		conf.Host,
		conf.Username,
		conf.Password,
		conf.Name,
		conf.Port,
	)
	// create open database connection
	db, err := gorm.Open(postgres.Open(dsn), &gorm.Config{})
	if err != nil {
		return nil, err
	}
	return db, nil
}

func Automigrate(db *gorm.DB) error {
	return db.AutoMigrate(
		&types.Barcode{},
		&types.BundleData{},
		&types.ProductionOrder{},
		&types.CustomerOrder{},
		&types.BeamCutPattern{},
		&types.LayerCutPattern{},
		&types.BeamGroup{},
	)
}

func New(conf *DBConfig) (Storager, error) {
	// database holder
	var db *gorm.DB
	// producctio
	db, err := ProductionDataBase(conf)
	if err != nil {
		return nil, err
	}
	// auto migration
	if err := Automigrate(db); err != nil {
		return nil, err
	}
	// done
	return &storage{
		db:  db,
		mux: sync.RWMutex{},
	}, nil
}
