package app

import (
	"context"
	"os"
	"os/signal"
	"syscall"

	"git.espin.casa/albert/cml04-falcon-system/labeler/server"
	"git.espin.casa/albert/cml04-falcon-system/labeler/service"
	"git.espin.casa/albert/logger"
	"github.com/spf13/cobra"
)

func Run(cmd *cobra.Command, args []string) {
	// read flags
	logLevel, _ := cmd.Flags().GetString("log-level")
	userName, _ := cmd.Flags().GetString("db-username")
	userPass, _ := cmd.Flags().GetString("db-password")
	dbHost, _ := cmd.Flags().GetString("db-host")
	dbPort, _ := cmd.Flags().GetInt("db-port")
	dbName, _ := cmd.Flags().GetString("db-name")
	natsHost, _ := cmd.Flags().GetString("nats-host")
	natsPort, _ := cmd.Flags().GetInt("nats-port")
	// create context
	ctx, cancel := context.WithCancel(context.Background())
	defer cancel()
	// setup logger
	log := logger.New(os.Stdout, logLevel)
	// log fields
	logFields := logger.LogFields{
		"nats_host": natsHost,
		"nats_port": natsPort,
		"log_level": logLevel,
	}
	// create service
	svc, err := service.New(&service.DBConfig{
		Username: userName,
		Password: userPass,
		Host:     dbHost,
		Port:     dbPort,
		Name:     dbName,
	}, &service.NATSconfig{
		Host: natsHost,
		Port: natsPort,
	})
	// check error
	if err != nil {
		log.Error("create service failed", err, logFields)
		return
	}
	defer svc.Close(ctx)
	// create server
	server := server.New("", svc, log)
	server.Start()
	// info banner
	log.Info("started cml04-falcon-labeler service", logFields)
	// wait signal to finish
	signal := WaitSignal()
	log.Info("signal received", logFields.Add(logger.LogFields{
		"signal": signal,
	}))
}

// WaitSignal catching exit signal
func WaitSignal() os.Signal {
	ch := make(chan os.Signal, 2)
	signal.Notify(
		ch,
		syscall.SIGINT,
		syscall.SIGQUIT,
		syscall.SIGTERM,
	)
	for {
		sig := <-ch
		switch sig {
		case syscall.SIGINT, syscall.SIGQUIT, syscall.SIGTERM:
			return sig
		}
	}
}
